"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generate = void 0;
const tslib_1 = require("tslib");
const typescript_1 = (0, tslib_1.__importDefault)(require("typescript"));
const fs = (0, tslib_1.__importStar)(require("fs"));
const path = (0, tslib_1.__importStar)(require("path"));
const chokidar = (0, tslib_1.__importStar)(require("chokidar"));
const fs_extra_1 = (0, tslib_1.__importDefault)(require("fs-extra"));
const fast_glob_1 = (0, tslib_1.__importDefault)(require("fast-glob"));
const core_1 = require("./core");
const typescriptGenerator = (0, tslib_1.__importStar)(require("./typescript-generator"));
(0, tslib_1.__exportStar)(require("./core"), exports);
async function generate(configuration) {
    function generateJsonSchemas(generator) {
        if (!configuration.jsonSchemaOutputDirectory) {
            return;
        }
        fs_extra_1.default.ensureDirSync(configuration.jsonSchemaOutputDirectory);
        const schemas = generator.generateJsonSchemas();
        for (const { entry, schema } of schemas) {
            if (entry) {
                fs.writeFileSync(path.resolve(configuration.jsonSchemaOutputDirectory, entry), JSON.stringify(schema, null, '  '));
            }
            else {
                console.log(`json schema verified fail for entry: ${entry}`);
                if (!configuration.watch) {
                    process.exit(1);
                }
            }
        }
    }
    let program;
    async function run() {
        const files = await (0, fast_glob_1.default)(configuration.files);
        const newProgram = typescript_1.default.createProgram(files, { target: typescript_1.default.ScriptTarget.ESNext }, undefined, program);
        program = newProgram;
        const sourceFiles = files.map(filePath => newProgram.getSourceFile(filePath));
        const generator = new core_1.Generator(sourceFiles.filter((s) => !!s), configuration.loose ?? false, !!configuration.plugins, fileName => path.relative(process.cwd(), fileName), program.getTypeChecker());
        if (configuration.debugOutputPath) {
            fs.writeFileSync(configuration.debugOutputPath, JSON.stringify(generator.declarations, null, '  '));
        }
        if (configuration.protobufOutputPath) {
            const protobufContent = `/**
 * This file is generated by 'types-as-schema'
 * It is not mean to be edited by hand
 */
` + generator.generateProtobuf();
            fs.writeFileSync(configuration.protobufOutputPath, protobufContent);
        }
        if (configuration.swagger) {
            let swaggerBase;
            if (typeof configuration.swagger.base === 'string') {
                swaggerBase = JSON.parse(fs.readFileSync(configuration.swagger.base).toString());
            }
            else if (configuration.swagger.base) {
                swaggerBase = configuration.swagger.base;
            }
            const swaggerDoc = generator.generateSwaggerDoc(swaggerBase);
            fs.writeFileSync(configuration.swagger.outputPath, swaggerDoc);
        }
        if (configuration.jsonSchemaOutputDirectory) {
            generateJsonSchemas(generator);
        }
        if (configuration.plugins) {
            for (const plugin of configuration.plugins) {
                let action;
                if (typeof plugin === 'string') {
                    let configFilePath;
                    if (path.isAbsolute(plugin)) {
                        configFilePath = plugin;
                    }
                    else if (plugin.startsWith(`.${path.sep}`)
                        || plugin.startsWith(`..${path.sep}`)
                        || plugin.startsWith('./')
                        || plugin.startsWith('../')) {
                        configFilePath = path.resolve(process.cwd(), plugin);
                    }
                    else {
                        configFilePath = plugin;
                    }
                    if (configFilePath.endsWith('.ts')) {
                        require('ts-node/register/transpile-only');
                    }
                    // eslint-disable-next-line @typescript-eslint/no-var-requires
                    const actionFromFile = require(configFilePath);
                    if (actionFromFile.default) {
                        action = actionFromFile.default;
                    }
                    else {
                        action = actionFromFile;
                    }
                }
                else {
                    action = plugin;
                }
                const customContent = action(generator.declarations, typescriptGenerator);
                if (typeof customContent === 'string' && configuration.customPath) {
                    fs.writeFileSync(configuration.customPath, customContent);
                }
                else if (Array.isArray(customContent)) {
                    for (const r of customContent) {
                        fs.writeFileSync(r.path, r.content);
                    }
                }
            }
        }
        if (configuration.typescriptOutputPath) {
            const typescriptContent = generator.generateTypescript();
            fs.writeFileSync(configuration.typescriptOutputPath, typescriptContent);
        }
        if (configuration.markdownOutputPath) {
            const markdownContent = generator.generateMarkdownDoc();
            fs.writeFileSync(configuration.markdownOutputPath, markdownContent);
        }
    }
    if (configuration.watch) {
        let ready = false;
        chokidar.watch(configuration.files).on('ready', () => {
            ready = true;
            run();
        }).on('all', (type, file) => {
            console.log(`Detecting ${type}: ${file}`);
            if (ready) {
                run();
            }
        });
    }
    else {
        await run();
    }
}
exports.generate = generate;
