export declare function warn(position: PositionValue, stage: string): void;
export declare function isValidReference(declarations: TypeDeclaration[], referenceName: string): boolean;
export declare function getReferencesInType(type: Type): ReferenceType[];
export interface Context {
    looseMode: boolean;
    declarations: TypeDeclaration[];
    allowFileType?: boolean;
}
export declare type TypeDeclaration = EnumDeclaration | ObjectDeclaration | ArrayDeclaration | UnionDeclaration | StringDeclaration | NumberDeclaration | BooleanDeclaration | MapDeclaration | ReferenceDeclaration | FunctionDeclaration | NullDeclaration | UnknownDeclaration;
/**
 * @public
 */
export declare type UnknownDeclaration = {
    kind: 'unknown';
    description?: string;
} & Position & JsDocAndComment & Modifiers & TypeArguments & DeclarationCommon;
export declare type EnumDeclaration = DeclarationCommon & {
    kind: 'enum';
    type: string;
    members: EnumMember[];
    description?: string;
} & Position & JsDocAndComment & Modifiers;
/**
 * @public
 */
export interface Position {
    position: PositionValue;
}
/**
 * @public
 */
export interface PositionValue {
    file: string;
    line: number;
    character: number;
}
/**
 * @public
 */
export interface Modifiers {
    modifiers?: Modifier[];
}
/**
 * @public
 */
export declare type Modifier = 'abstract' | 'async' | 'const' | 'declare' | 'default' | 'export' | 'private' | 'protected' | 'public' | 'override' | 'readonly' | 'static';
/**
 * @public
 */
export interface EnumMember {
    name: string;
    value: string | number;
    description?: string;
}
export declare type ObjectDeclaration = ObjectType & Modifiers & DeclarationCommon;
export declare type ArrayDeclaration = ArrayType & Modifiers & DeclarationCommon;
export declare type UnionDeclaration = UnionType & Modifiers & DeclarationCommon & {
    objectType?: ObjectType;
};
export declare type StringDeclaration = StringType & Modifiers & DeclarationCommon;
export declare type NumberDeclaration = NumberType & Modifiers & DeclarationCommon;
/**
 * @public
 */
export declare type BooleanDeclaration = BooleanType & Modifiers & DeclarationCommon;
/**
 * @public
 */
export declare type NullDeclaration = NullType & Modifiers & DeclarationCommon;
/**
 * @public
 */
export declare type MapDeclaration = MapType & Modifiers & DeclarationCommon;
export declare type ReferenceDeclaration = ReferenceType & Modifiers & DeclarationCommon;
interface DeclarationCommon {
    name: string;
    entry?: string;
}
export declare type FunctionDeclaration = FunctionType & Modifiers & JsDocAndComment & TypeArguments & TypeParameters & DeclarationCommon & {
    optional?: boolean;
    method?: string;
    path?: string;
    summary?: string;
    deprecated?: boolean;
    tags?: string[];
    body?: string;
};
/**
 * @public
 */
export interface FunctionType extends Position {
    kind: 'function';
    type: Type;
    parameters: FunctionParameter[];
    default?: unknown;
    description?: string;
}
export declare type FunctionParameter = Parameter & {
    in?: string;
};
export declare type Type = StringType | MapType | ArrayType | EnumType | ReferenceType | ObjectType | NumberType | BooleanType | AnyType | NullType | UnionType | FileType | VoidType | FunctionType;
export declare type MapType = {
    kind: 'map';
    key: Type;
    value: Type;
    default?: unknown;
    description?: string;
} & Position & JsDocAndComment;
/**
 * @public
 */
export declare type EnumType = {
    kind: 'enum';
    type: string;
    name: string;
    enums: unknown[];
    default?: unknown;
    description?: string;
} & Position & JsDocAndComment;
export declare type ReferenceType = {
    kind: 'reference';
    referenceName: string;
    default?: unknown;
    description?: string;
} & Position & JsDocAndComment;
export declare type NumberType = {
    kind: 'number';
    type: string;
    minimum?: number;
    maximum?: number;
    exclusiveMinimum?: number;
    exclusiveMaximum?: number;
    multipleOf?: number;
    default?: number;
    enums?: string[];
    title?: string;
    description?: string;
} & Position & JsDocAndComment;
export declare type StringType = {
    kind: 'string';
    minLength?: number;
    maxLength?: number;
    pattern?: string;
    default?: string;
    enums?: string[];
    title?: string;
    description?: string;
    lowercase?: boolean;
    uppercase?: boolean;
    trim?: boolean;
    templateLiteral?: TemplateLiteralPart[];
} & Position & JsDocAndComment;
export declare type TemplateLiteralPart = {
    kind: 'enum';
    enums: string[];
} | {
    kind: 'string';
} | {
    kind: 'number';
} | {
    kind: 'boolean';
};
export declare function templateLiteralToPattern(templateLiteral: TemplateLiteralPart[]): string;
export declare type BooleanType = {
    kind: 'boolean';
    default?: boolean;
    title?: string;
    description?: string;
} & Position & JsDocAndComment;
/**
 * @public
 */
export declare type AnyType = {
    kind: undefined;
    default?: unknown;
    description?: string;
} & Position & JsDocAndComment;
export declare type ObjectType = {
    kind: 'object';
    members: Member[];
    minProperties: number;
    maxProperties?: number;
    additionalProperties?: boolean | Type;
    default?: unknown;
    title?: string;
    description?: string;
} & Position & JsDocAndComment & Decorators;
export declare type Decorator = {
    name: string;
    parameters?: unknown[];
};
declare type Decorators = {
    decorators?: Decorator[];
};
export declare type ArrayType = {
    kind: 'array';
    type: Type;
    uniqueItems?: boolean;
    minItems?: number;
    maxItems?: number;
    default?: unknown[];
    title?: string;
    description?: string;
} & Position & JsDocAndComment;
/**
 * @public
 */
export declare type NullType = {
    kind: 'null';
    default?: unknown;
    description?: string;
} & Position & JsDocAndComment;
/**
 * @public
 */
export declare type FileType = {
    kind: 'file';
    default?: unknown;
    description?: string;
} & Position;
/**
 * @public
 */
export declare type VoidType = {
    kind: 'void';
    default?: unknown;
    description?: string;
} & Position;
/**
 * @public
 */
export declare type UnionType = {
    kind: 'union';
    members: Type[];
    default?: unknown;
    description?: string;
} & Position & JsDocAndComment;
export interface Member extends JsDocAndComment, Decorators {
    name: string;
    type: Type;
    optional?: boolean;
    tag?: number;
    index?: boolean;
    unique?: boolean;
    sparse?: boolean;
    select?: boolean;
    alias?: string;
    parameters?: Parameter[];
}
export interface JsDocAndComment {
    comments?: string[];
    jsDocs?: JsDoc[];
}
export interface JsDoc {
    name: string;
    type?: Type;
    paramName?: string;
    comment?: string;
    optional?: boolean;
}
/**
 * @public
 */
export interface Parameter extends JsDocAndComment, Decorators {
    name: string;
    type: Type;
    optional?: boolean;
}
export interface Expression {
    name: string;
    type: Type;
    value: unknown;
}
/**
 * @public
 */
export interface TypeArguments {
    typeArguments?: Type[];
}
/**
 * @public
 */
export interface TypeParameters {
    typeParameters?: TypeParameter[];
}
/**
 * @public
 */
export interface TypeParameter {
    name: string;
    constraint?: Type;
}
export {};
